/****************************************************************************
** Meta object code from reading C++ file 'richtextcomposercontroler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kpimtextedit-26.04.2/src/composer-ng/richtextcomposercontroler.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'richtextcomposercontroler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KPIMTextEdit25RichTextComposerControlerE_t {};
} // unnamed namespace

template <> constexpr inline auto KPIMTextEdit::RichTextComposerControler::qt_create_metaobjectdata<qt_meta_tag_ZN12KPIMTextEdit25RichTextComposerControlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPIMTextEdit::RichTextComposerControler",
        "addCheckbox",
        "",
        "add",
        "insertHorizontalRule",
        "setHeadingLevel",
        "level",
        "alignLeft",
        "alignCenter",
        "alignRight",
        "alignJustify",
        "makeRightToLeft",
        "makeLeftToRight",
        "setTextBold",
        "bold",
        "setTextItalic",
        "italic",
        "setTextUnderline",
        "underline",
        "setTextStrikeOut",
        "strikeOut",
        "setTextForegroundColor",
        "QColor",
        "color",
        "setTextBackgroundColor",
        "setFontFamily",
        "fontFamily",
        "setFontSize",
        "size",
        "setFont",
        "QFont",
        "font",
        "setTextSuperScript",
        "superscript",
        "setTextSubScript",
        "subscript",
        "setChangeTextForegroundColor",
        "setChangeTextBackgroundColor",
        "manageLink",
        "indentListMore",
        "indentListLess",
        "setListStyle",
        "styleIndex",
        "slotAddEmoticon",
        "text",
        "slotInsertHtml",
        "slotFormatReset",
        "slotPasteWithoutFormatting",
        "slotPasteAsQuotation",
        "slotRemoveQuotes",
        "slotAddQuotes",
        "slotAddImage",
        "slotFormatPainter",
        "active",
        "ensureCursorVisibleDelayed"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addCheckbox'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'addCheckbox'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'insertHorizontalRule'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setHeadingLevel'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'alignLeft'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'alignCenter'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'alignRight'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'alignJustify'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'makeRightToLeft'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'makeLeftToRight'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setTextBold'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setTextItalic'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setTextUnderline'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'setTextStrikeOut'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'setTextForegroundColor'
        QtMocHelpers::SlotData<void(const QColor &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'setTextBackgroundColor'
        QtMocHelpers::SlotData<void(const QColor &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'setFontFamily'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Slot 'setFont'
        QtMocHelpers::SlotData<void(const QFont &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'setTextSuperScript'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 33 },
        }}),
        // Slot 'setTextSubScript'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'setChangeTextForegroundColor'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setChangeTextBackgroundColor'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'manageLink'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'indentListMore'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'indentListLess'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setListStyle'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 42 },
        }}),
        // Slot 'slotAddEmoticon'
        QtMocHelpers::SlotData<void(const QString &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Slot 'slotInsertHtml'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFormatReset'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPasteWithoutFormatting'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPasteAsQuotation'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRemoveQuotes'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddQuotes'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddImage'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFormatPainter'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 53 },
        }}),
        // Slot 'ensureCursorVisibleDelayed'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RichTextComposerControler, qt_meta_tag_ZN12KPIMTextEdit25RichTextComposerControlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPIMTextEdit::RichTextComposerControler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KPIMTextEdit25RichTextComposerControlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KPIMTextEdit25RichTextComposerControlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KPIMTextEdit25RichTextComposerControlerE_t>.metaTypes,
    nullptr
} };

void KPIMTextEdit::RichTextComposerControler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RichTextComposerControler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addCheckbox((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->addCheckbox(); break;
        case 2: _t->insertHorizontalRule(); break;
        case 3: _t->setHeadingLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->alignLeft(); break;
        case 5: _t->alignCenter(); break;
        case 6: _t->alignRight(); break;
        case 7: _t->alignJustify(); break;
        case 8: _t->makeRightToLeft(); break;
        case 9: _t->makeLeftToRight(); break;
        case 10: _t->setTextBold((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setTextItalic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setTextUnderline((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->setTextStrikeOut((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setTextForegroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->setTextBackgroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 16: _t->setFontFamily((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->setFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 19: _t->setTextSuperScript((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->setTextSubScript((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->setChangeTextForegroundColor(); break;
        case 22: _t->setChangeTextBackgroundColor(); break;
        case 23: _t->manageLink(); break;
        case 24: _t->indentListMore(); break;
        case 25: _t->indentListLess(); break;
        case 26: _t->setListStyle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->slotAddEmoticon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->slotInsertHtml(); break;
        case 29: _t->slotFormatReset(); break;
        case 30: _t->slotPasteWithoutFormatting(); break;
        case 31: _t->slotPasteAsQuotation(); break;
        case 32: _t->slotRemoveQuotes(); break;
        case 33: _t->slotAddQuotes(); break;
        case 34: _t->slotAddImage(); break;
        case 35: _t->slotFormatPainter((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->ensureCursorVisibleDelayed(); break;
        default: ;
        }
    }
}

const QMetaObject *KPIMTextEdit::RichTextComposerControler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPIMTextEdit::RichTextComposerControler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KPIMTextEdit25RichTextComposerControlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KPIMTextEdit::RichTextComposerControler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 37;
    }
    return _id;
}
QT_WARNING_POP
