/****************************************************************************
** Meta object code from reading C++ file 'richtextcomposer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kpimtextedit-26.04.2/src/composer-ng/richtextcomposer.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'richtextcomposer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KPIMTextEdit16RichTextComposerE_t {};
} // unnamed namespace

template <> constexpr inline auto KPIMTextEdit::RichTextComposer::qt_create_metaobjectdata<qt_meta_tag_ZN12KPIMTextEdit16RichTextComposerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KPIMTextEdit::RichTextComposer",
        "insertModeChanged",
        "",
        "textModeChanged",
        "KPIMTextEdit::RichTextComposer::Mode",
        "mode",
        "focusUp",
        "externalEditorStarted",
        "externalEditorClosed",
        "insertEmoticon",
        "str",
        "insertPlainTextImplementation",
        "slotChangeInsertMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'insertModeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'textModeChanged'
        QtMocHelpers::SignalData<void(KPIMTextEdit::RichTextComposer::Mode)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'focusUp'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'externalEditorStarted'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'externalEditorClosed'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'insertEmoticon'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'insertPlainTextImplementation'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChangeInsertMode'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RichTextComposer, qt_meta_tag_ZN12KPIMTextEdit16RichTextComposerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KPIMTextEdit::RichTextComposer::staticMetaObject = { {
    QMetaObject::SuperData::link<TextCustomEditor::RichTextEditor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KPIMTextEdit16RichTextComposerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KPIMTextEdit16RichTextComposerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KPIMTextEdit16RichTextComposerE_t>.metaTypes,
    nullptr
} };

void KPIMTextEdit::RichTextComposer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RichTextComposer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->insertModeChanged(); break;
        case 1: _t->textModeChanged((*reinterpret_cast<std::add_pointer_t<KPIMTextEdit::RichTextComposer::Mode>>(_a[1]))); break;
        case 2: _t->focusUp(); break;
        case 3: _t->externalEditorStarted(); break;
        case 4: _t->externalEditorClosed(); break;
        case 5: _t->insertEmoticon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->insertPlainTextImplementation(); break;
        case 7: _t->slotChangeInsertMode(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RichTextComposer::*)()>(_a, &RichTextComposer::insertModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RichTextComposer::*)(KPIMTextEdit::RichTextComposer::Mode )>(_a, &RichTextComposer::textModeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RichTextComposer::*)()>(_a, &RichTextComposer::focusUp, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RichTextComposer::*)()>(_a, &RichTextComposer::externalEditorStarted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RichTextComposer::*)()>(_a, &RichTextComposer::externalEditorClosed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (RichTextComposer::*)(const QString & )>(_a, &RichTextComposer::insertEmoticon, 5))
            return;
    }
}

const QMetaObject *KPIMTextEdit::RichTextComposer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KPIMTextEdit::RichTextComposer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KPIMTextEdit16RichTextComposerE_t>.strings))
        return static_cast<void*>(this);
    return TextCustomEditor::RichTextEditor::qt_metacast(_clname);
}

int KPIMTextEdit::RichTextComposer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TextCustomEditor::RichTextEditor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void KPIMTextEdit::RichTextComposer::insertModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KPIMTextEdit::RichTextComposer::textModeChanged(KPIMTextEdit::RichTextComposer::Mode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KPIMTextEdit::RichTextComposer::focusUp()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KPIMTextEdit::RichTextComposer::externalEditorStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KPIMTextEdit::RichTextComposer::externalEditorClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KPIMTextEdit::RichTextComposer::insertEmoticon(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
